#ifdef USE_LIGHTS

#define MAX_LIGHTS 8

#define LIGHT_OMNI 0
#define LIGHT_OMNINORMAL 1
#define LIGHT_SPOT 2
#define LIGHT_SPOTNORMAL 3
#define LIGHT_SHADOW 4

// lights
uniform int numLights;
uniform vec3 lightPos[MAX_LIGHTS];
uniform vec3 lightColor[MAX_LIGHTS];
uniform vec3 lightParams[MAX_LIGHTS];
uniform mat3 lightDir[MAX_LIGHTS];

#ifdef SHADOW_BOX
// shadow box
uniform bool modelShadow;
uniform vec3 shadowColor;
#endif  // SHADOW_BOX

void ProcessLight(vec3 lightPos, vec3 lightColor, vec3 lightParams, mat3 lightDir, inout bool inShadow)
{
  int type = int(lightParams.x);
  vec3 delta = lightPos - inPosition.xyz;

#ifdef SHADOW_BOX
  // special case for shadow box
  if (type == LIGHT_SHADOW) {
    vec3 dist = delta * lightDir;
    vec3 lightMin = -1.0 * lightColor;
    vec3 lightMax = lightColor;
    if (all(greaterThan(dist, lightMin)) && 
        all(lessThan(dist, lightMax))) {
      inShadow = true;
    }

  // other lights
  } else
#endif  // SHADOW_BOX
  {
    float reach = lightParams.y;
    float dist = dot(delta, delta);
    if (dist < reach) {
      float scale = 1.0 - (dist / reach);
      dist = sqrt(dist);

      // calc angle from normal if needed
      if (type == LIGHT_OMNINORMAL || type == LIGHT_SPOTNORMAL) {
        float ang = dot(delta, inNormal);
        ang = clamp(ang / dist, 0.0, 1.0);
        scale *= ang;
      }

      // calc cone adjustment if needed
      if (type == LIGHT_SPOT || type == LIGHT_SPOTNORMAL) {
        float conemul = lightParams.z;
        float cone = -dot(lightDir[2], delta) / dist - 1.0;
        cone = cone * conemul + 1.0;
        cone = clamp(cone, 0.0, 1.0);
        scale *= cone;
      }

      varColor.rgb += mix(vec3(0.0), lightColor, scale);
    }
  }
}

void ProcessLights(void)
{
#ifdef SHADOW_BOX
  bool inShadow = modelShadow;
#else
  bool inShadow = false;
#endif  // SHADOW_BOX

  if (numLights > 0) {
    ProcessLight(lightPos[0], lightColor[0], lightParams[0], lightDir[0], inShadow);
  }
  if (numLights > 1) {
    ProcessLight(lightPos[1], lightColor[1], lightParams[1], lightDir[1], inShadow);
  }
  if (numLights > 2) {
    ProcessLight(lightPos[2], lightColor[2], lightParams[2], lightDir[2], inShadow);
  }
  if (numLights > 3) {
    ProcessLight(lightPos[3], lightColor[3], lightParams[3], lightDir[3], inShadow);
  }
  if (numLights > 4) {
    ProcessLight(lightPos[4], lightColor[4], lightParams[4], lightDir[4], inShadow);
  }
  if (numLights > 5) {
    ProcessLight(lightPos[5], lightColor[5], lightParams[5], lightDir[5], inShadow);
  }
  if (numLights > 6) {
    ProcessLight(lightPos[6], lightColor[6], lightParams[6], lightDir[6], inShadow);
  }
  if (numLights > 7) {
    ProcessLight(lightPos[7], lightColor[7], lightParams[7], lightDir[7], inShadow);
  }

#ifdef SHADOW_BOX
  if (inShadow) {
    varColor.rgb += shadowColor;
  }
#endif  // SHADOW_BOX
}
#endif  // USE_LIGHTS
